@@ -28,6 +28,7 @@
 #define __STDC_WANT_MATH_SPEC_FUNCS__ 1
 #define _TANDEM_SOURCE 1
 #define __EXTENSIONS__ 1
+#define RB_THREAD_LOCAL_SPECIFIER_IS_UNSUPPORTED 1
 #define RUBY_SYMBOL_EXPORT_BEGIN _Pragma("GCC visibility push(default)")
 #define RUBY_SYMBOL_EXPORT_END _Pragma("GCC visibility pop")
 #define HAVE_STMT_AND_DECL_IN_EXPR 1
@@ -231,7 +232,6 @@
 #define HAVE_VFORK 1
 #define HAVE_WORKING_VFORK 1
 #define HAVE_WORKING_FORK 1
-#define HAVE___SYSCALL 1
 #define HAVE__LONGJMP 1
 #define HAVE_ARC4RANDOM_BUF 1
 #define HAVE_ATAN2L 1
