@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("3.0.") or
-    raise "ruby lib version (3.0.5) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (3.0.6) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.0/aarch64-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "3"
   CONFIG["MINOR"] = "0"
-  CONFIG["TEENY"] = "5"
-  CONFIG["PATCHLEVEL"] = "211"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.0.5/bin/install -c'
+  CONFIG["TEENY"] = "6"
+  CONFIG["PATCHLEVEL"] = "216"
+  CONFIG["INSTALL"] = '/build/tmp/pobj//ruby-3.0.6/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)30"
@@ -150,9 +150,9 @@ module RbConfig
   CONFIG["DOT"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.0.5/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.0.5/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.0.5/bin/install -c  -m 755"
+  CONFIG["INSTALL_DATA"] = "/build/tmp/pobj//ruby-3.0.6/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/build/tmp/pobj//ruby-3.0.6/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/build/tmp/pobj//ruby-3.0.6/bin/install -c  -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
@@ -202,7 +202,7 @@ module RbConfig
   CONFIG["RUBY_BASE_NAME"] = "ruby"
   CONFIG["RUBY_PROGRAM_VERSION"] = "$(MAJOR).$(MINOR).$(TEENY)"
   CONFIG["RUBY_API_VERSION"] = "$(MAJOR).$(MINOR)"
-  CONFIG["HAVE_GIT"] = "no"
+  CONFIG["HAVE_GIT"] = "yes"
   CONFIG["GIT"] = "git"
   CONFIG["cxxflags"] = ""
   CONFIG["cppflags"] = ""
