@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module Fiddle
   class Function
     # The ABI of the Function.
@@ -9,6 +10,11 @@ module Fiddle
     # The name of this function
     attr_reader :name
 
+    # Whether GVL is needed to call this function
+    def need_gvl?
+      @need_gvl
+    end
+
     # The integer memory location of this function
     def to_i
       ptr.to_i
