@@ -16,6 +16,23 @@
 # * /dev/urandom
 # * Win32
 #
+# SecureRandom is extended by the Random::Formatter module which
+# defines the following methods:
+#
+# * alphanumeric
+# * base64
+# * choose
+# * gen_random
+# * hex
+# * rand
+# * random_bytes
+# * random_number
+# * urlsafe_base64
+# * uuid
+#
+# These methods are usable as class methods of SecureRandom such as
+# `SecureRandom.hex`.
+#
 # === Examples
 #
 # Generate random hexadecimal strings:
@@ -49,40 +66,11 @@
 #
 
 module SecureRandom
-  @rng_chooser = Mutex.new # :nodoc:
-
   class << self
     def bytes(n)
       return gen_random(n)
     end
 
-    def gen_random(n)
-      ret = Random.urandom(1)
-      if ret.nil?
-        begin
-          require 'openssl'
-        rescue NoMethodError
-          raise NotImplementedError, "No random device"
-        else
-          @rng_chooser.synchronize do
-            class << self
-              remove_method :gen_random
-              alias gen_random gen_random_openssl
-            end
-          end
-          return gen_random(n)
-        end
-      else
-        @rng_chooser.synchronize do
-          class << self
-            remove_method :gen_random
-            alias gen_random gen_random_urandom
-          end
-        end
-        return gen_random(n)
-      end
-    end
-
     private
 
     def gen_random_openssl(n)
@@ -110,6 +98,21 @@ module SecureRandom
       end
       ret
     end
+
+    ret = Random.urandom(1)
+    if ret.nil?
+      begin
+        require 'openssl'
+      rescue NoMethodError
+        raise NotImplementedError, "No random device"
+      else
+        alias gen_random gen_random_openssl
+      end
+    else
+      alias gen_random gen_random_urandom
+    end
+
+    public :gen_random
   end
 end
 
